# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: util.py
@time: 2019/12/19 11:03

这一行开始写关于本文件的说明与解释


"""
import importlib
import json
import logging
import os
import pkgutil
import random
import subprocess
import sys
# import torch.distributed as dist
import tensorflow as tf
from itertools import zip_longest, islice
from logging import Filter
from typing import Any, Callable, Dict, List, Tuple, TypeVar, Iterable, Iterator

try:
    import resource
except ImportError:
    # resource doesn't exist on Windows systems
    resource = None

# import torch
import numpy

logger = logging.getLogger(__name__)

JsonDict = Dict[str, Any]

# If you want to have start and/or end symbols for any reason in your code, we recommend you use
# these, to have a common place to import from.  Also, it's important for some edge cases in how
# data is processed for these symbols to be lowercase, not uppercase (because we have code that
# will lowercase tokens for you in some circumstances, and we need this symbol to not change in
# those cases).
START_SYMBOL = "@start@"
END_SYMBOL = "@end@"


def group_by_count(iterable: List[Any], count: int, default_value: Any) -> List[List[Any]]:
    """
    Takes a list and groups it into sub lists of size ``count``, using ``default_value`` to pad the
    list at the end if the list is not divis able by ``count``.
    For example:
    >>> group_by_count([1, 2, 3, 4, 5, 6, 7], 3, 0)
    [[1, 2, 3], [4, 5, 6], [7, 0, 0]]
    This is a short method, but it's complicated and hard to remember as a one-liner, so we just
    make a function out of it.
    """
    return [list(l) for l in zip_longest(*[iter(iterable)] * count, fillvalue=default_value)]


A = TypeVar("A")


def lazy_groups_of(iterable: Iterable[A], group_size: int) -> Iterator[List[A]]:
    """
    Takes an iterable and batches the individual instances into lists of the
    specified size. The last list may be smaller if there are instances left over.
    """
    iterator = iter(iterable)
    while True:
        s = list(islice(iterator, group_size))
        if len(s) > 0:
            yield s
        else:
            break


def pad_sequence_to_length(
        sequence: List,
        desired_length: int,
        default_value: Callable[[], Any] = lambda: 0,
        padding_on_right: bool = True,
) -> List:
    """
    Take a list of objects and pads it to the desired length, returning the padded list.  The
    original list is not modified.
    Parameters
    ----------
    sequence : List
        A list of objects to be padded.
    desired_length : int
        Maximum length of each sequence. Longer sequences are truncated to this length, and
        shorter ones are padded to it.
    default_value: Callable, default=lambda: 0
        Callable that outputs a default value (of any type) to use as padding values.  This is
        a lambda to avoid using the same object when the default value is more complex, like a
        list.
    padding_on_right : bool, default=True
        When we add padding tokens (or truncate the sequence), should we do it on the right or
        the left?
    Returns
    -------
    padded_sequence : List
    """
    # Truncates the sequence to the desired length.
    if padding_on_right:
        padded_sequence = sequence[:desired_length]
    else:
        padded_sequence = sequence[-desired_length:]
    # Continues to pad with default_value() until we reach the desired length.
    for _ in range(desired_length - len(padded_sequence)):
        if padding_on_right:
            padded_sequence.append(default_value())
        else:
            padded_sequence.insert(0, default_value())
    return padded_sequence


def add_noise_to_dict_values(dictionary: Dict[A, float], noise_param: float) -> Dict[A, float]:
    """
    Returns a new dictionary with noise added to every key in ``dictionary``.  The noise is
    uniformly distributed within ``noise_param`` percent of the value for every value in the
    dictionary.
    """
    new_dict = {}
    for key, value in dictionary.items():
        noise_value = value * noise_param
        noise = random.uniform(-noise_value, noise_value)
        new_dict[key] = value + noise
    return new_dict


def namespace_match(pattern: str, namespace: str):
    """
    Matches a namespace pattern against a namespace string.  For example, ``*tags`` matches
    ``passage_tags`` and ``question_tags`` and ``tokens`` matches ``tokens`` but not
    ``stemmed_tokens``.
    """
    if pattern[0] == "*" and namespace.endswith(pattern[1:]):
        return True
    elif pattern == namespace:
        return True
    return False


class FileFriendlyLogFilter(Filter):
    """
    TQDM and requests use carriage returns to get the training line to update for each batch
    without adding more lines to the terminal output.  Displaying those in a file won't work
    correctly, so we'll just make sure that each batch shows up on its one line.
    """

    def filter(self, record):
        if "\r" in record.msg:
            record.msg = record.msg.replace("\r", "")
            if not record.msg or record.msg[-1] != "\n":
                record.msg += "\n"
        return True


class WorkerLogFilter(Filter):
    def __init__(self, rank=-1):
        super().__init__()
        self._rank = rank

    def filter(self, record):
        if self._rank != -1:
            record.msg = f"Rank {self._rank} | {record.msg}"
        return True


def import_submodules(package_name: str) -> None:
    """
    Import all submodules under the given package.
    Primarily useful so that people using AllenNLP as a library
    can specify their own custom packages and have their custom
    classes get loaded and registered.
    """
    importlib.invalidate_caches()

    # For some reason, python doesn't always add this by default to your path, but you pretty much
    # always want it when using `--include-package`.  And if it's already there, adding it again at
    # the end won't hurt anything.
    sys.path.append(".")

    # Import at top level
    module = importlib.import_module(package_name)
    path = getattr(module, "__path__", [])
    path_string = "" if not path else path[0]

    # walk_packages only finds immediate children, so need to recurse.
    for module_finder, name, _ in pkgutil.walk_packages(path):
        # Sometimes when you import third-party libraries that are on your path,
        # `pkgutil.walk_packages` returns those too, so we need to skip them.
        if path_string and module_finder.path != path_string:
            continue
        subpackage = f"{package_name}.{name}"
        import_submodules(subpackage)


def peak_memory_mb() -> float:
    """
    Get peak memory usage for this process, as measured by
    max-resident-set size:
    https://unix.stackexchange.com/questions/30940/getrusage-system-call-what-is-maximum-resident-set-size
    Only works on OSX and Linux, returns 0.0 otherwise.
    """
    if resource is None or sys.platform not in ("linux", "darwin"):
        return 0.0

    # TODO(joelgrus): For whatever, our pinned version 0.521 of mypy does not like
    # next line, but later versions (e.g. 0.530) are fine with it. Once we get that
    # figured out, remove the type: ignore.
    peak = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss  # type: ignore

    if sys.platform == "darwin":
        # On OSX the result is in bytes.
        return peak / 1_000_000

    else:
        # On Linux the result is in kilobytes.
        return peak / 1_000


def gpu_memory_mb() -> Dict[int, int]:
    """
    Get the current GPU memory usage.
    Based on https://discuss.pytorch.org/t/access-gpu-memory-usage-in-pytorch/3192/4
    Returns
    -------
    ``Dict[int, int]``
        Keys are device ids as integers.
        Values are memory usage as integers in MB.
        Returns an empty ``dict`` if GPUs are not available.
    """
    try:
        result = subprocess.check_output(
            ["nvidia-smi", "--query-gpu=memory.used", "--format=csv,nounits,noheader"],
            encoding="utf-8",
        )
        gpu_memory = [int(x) for x in result.strip().split("\n")]
        return {gpu: memory for gpu, memory in enumerate(gpu_memory)}
    except FileNotFoundError:
        # `nvidia-smi` doesn't exist, assume that means no GPU.
        return {}
    except:  # noqa
        # Catch *all* exceptions, because this memory check is a nice-to-have
        # and we'd never want a training run to fail because of it.
        logger.exception("unable to check gpu_memory_mb(), continuing")
        return {}


def ensure_list(iterable: Iterable[A]) -> List[A]:
    """
    An Iterable may be a list or a generator.
    This ensures we get a list without making an unnecessary copy.
    """
    if isinstance(iterable, list):
        return iterable
    else:
        return list(iterable)


def is_lazy(iterable: Iterable[A]) -> bool:
    """
    Checks if the given iterable is lazy,
    which here just means it's not a list.
    """
    return not isinstance(iterable, list)


def get_frozen_and_tunable_parameter_names(model: tf.keras.models) -> List:
    frozen_parameter_names = []
    tunable_parameter_names = []
    for name in model.trainable_variables():
        frozen_parameter_names.append(name)
    for name in model.non_trainable_variables():
        tunable_parameter_names.append(name)
    return [frozen_parameter_names, tunable_parameter_names]


def dump_metrics(file_path: str, metrics: Dict[str, Any], log: bool = False) -> None:
    metrics_json = json.dumps(metrics, indent=2)
    with open(file_path, "w") as metrics_file:
        metrics_file.write(metrics_json)
    if log:
        logger.info("Metrics: %s", metrics_json)


def flatten_filename(file_path: str) -> str:
    return file_path.replace("/", "_SLASH_")


def is_master(rank: int = None, world_size: int = None) -> bool:
    """
    Checks if the process is a "master" in a distributed process group. If a
    process group is not initialized, this returns `True`.
    Parameters
    ----------
    rank : int ( default = None )
        Global rank of the process if in a distributed process group. If not
        given, rank is obtained using `torch.distributed.get_rank()`
    world_size : int ( default = None )
        Number of processes in the distributed group. If not
        given, this is obtained using `torch.distributed.get_world_size()`
    """
    distributed = dist.is_initialized()

    # In non-distributed case, a "master" process doesn't make any
    # sense. So instead of raising an error, returning True would
    # make things less painful
    if not distributed:
        return True

    if rank is None:
        rank = dist.get_rank()

    if world_size is None:
        world_size = dist.get_world_size()

    # rank == 0 would do in a single-node multi-GPU setup. However,
    # in a multi-node case, every node has a logical master and hence
    # the mod(%) op.
    return rank % world_size == 0


def is_distributed() -> bool:
    """
    Checks if the distributed process group has been initialized
    """
    return dist.is_initialized()



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: model.py
@time: 2019/12/19 17:12

这一行开始写关于本文件的说明与解释


"""
import tensorflow as tf

_DEFAULT_WEIGHTS = "best.th"


class Model(tf.keras.Model):
    def __init__(self, *args, **kwargs):
        super(Model, self).__init__(*args, **kwargs)

    def call(self, inputs, training=None, mask=None):
        raise NotImplementedError



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: checks.py
@time: 2019/12/11 15:06

这一行开始写关于本文件的说明与解释


"""


class ConfigurationError(Exception):
    """
    The exception raised by object when it's mis configured
    (e.g. missing properties, invalid properties, unknown properties).
    """

    def __init__(self, message):
        super().__init__()
        self.message = message

    def __str__(self):
        return repr(self.message)



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: bert_text_classifier.py
@time: 2019/12/10 14:31

这一行开始写关于本文件的说明与解释


"""

import codecs
import csv
import pickle

import tensorflow as tf
import collections
from bakend.bert import tokenization
from bakend.bert import modeling, optimization
import os

tf.logging.set_verbosity(tf.logging.INFO)


# os.environ['CUDA_VISIBLE_DEVICES'] = '1'

class DataProcessor(object):
    """Base class for data converters for sequence classification data sets."""

    def get_train_examples(self, data_dir):
        """Gets a collection of `InputExample`s for the train set."""
        raise NotImplementedError()

    def get_dev_examples(self, data_dir):
        """Gets a collection of `InputExample`s for the dev set."""
        raise NotImplementedError()

    def get_test_examples(self, data_dir):
        """Gets a collection of `InputExample`s for prediction."""
        raise NotImplementedError()

    def get_labels(self):
        """Gets the list of labels for this data set."""
        raise NotImplementedError()

    @classmethod
    def _read_tsv(cls, input_file, quotechar=None):
        """Reads a tab separated value file."""
        with tf.gfile.Open(input_file, "r") as f:
            reader = csv.reader(f, delimiter="\t", quotechar=quotechar)
            lines = []
            for line in reader:
                lines.append(line)
            return lines


class MyProcessor(DataProcessor):
    def __init__(self):
        super(MyProcessor).__init__()

    def get_train_examples(self, data_dir):
        with open(os.path.join(data_dir, "train.txt"), "r", encoding="utf-8") as f:
            lines = []
            for line in f:
                lines.append(line.replace("\n", "").strip())
        return self._create_examples(lines, "train")

    def get_dev_examples(self, data_dir):
        with open(os.path.join(data_dir, "eval.txt"), "r", encoding="utf-8") as f:
            lines = []
            for line in f:
                lines.append(line.replace("\n", "").strip())
        return self._create_examples(lines, "eval")

    def get_labels(self):
        """See base class."""
        return ["0", "1"]

    def _create_examples(self, lines, set_type):
        """Creates examples for the training and dev sets."""
        examples = []
        for (i, line) in enumerate(lines):
            # Only the test set has a header
            if set_type == "test" and i == 0:
                continue
            guid = "%s-%s" % (set_type, i)
            if set_type == "test":
                text_a = tokenization.convert_to_unicode(line[1])
                label = "0"
            else:
                text_a = tokenization.convert_to_unicode(line[0:-1].strip())
                label = tokenization.convert_to_unicode(line[-1].strip())
            examples.append(
                InputExample(guid=guid, text_a=text_a, text_b=None, label=label))
        return examples


class InputExample(object):
    """A single training/test example for simple sequence classification."""

    def __init__(self, guid, text_a, text_b=None, label=None):
        """Constructs a InputExample.

        Args:
          guid: Unique id for the example.
          text_a: string. The untokenized leslie of the first sequence. For single
            sequence tasks, only this sequence must be specified.
          text_b: (Optional) string. The untokenized leslie of the second sequence.
            Only must be specified for sequence pair tasks.
          label: (Optional) string. The label of the example. This should be
            specified for train and dev examples, but not for test examples.
        """
        self.guid = guid
        self.text_a = text_a
        self.text_b = text_b
        self.label = label


class InputFeatures(object):
    """A single set of features of data."""

    def __init__(self,
                 input_ids,
                 input_mask,
                 segment_ids,
                 label_id,
                 is_real_example=True):
        self.input_ids = input_ids
        self.input_mask = input_mask
        self.segment_ids = segment_ids
        self.label_id = label_id
        self.is_real_example = is_real_example


class BertClassifier:
    def __init__(self):
        self.data_root = './pretrained/chinese_L-12_H-768_A-12/'
        self.bert_config_file = self.data_root + 'bert_config.json'
        self.init_checkpoint = self.data_root + 'bert_model.ckpt'
        self.bert_vocab_file = self.data_root + 'vocab.txt'
        self.bert_config = modeling.BertConfig.from_json_file(self.bert_config_file)
        self.tokenizer = tokenization.FullTokenizer(vocab_file=self.bert_vocab_file)
        self.output_dir = ''

    def conver_single_example(self, ex_index, example, label_list, max_seq_length, tokenizer, mode):
        """
        将一个训练样本转化为InputFeature，其中进行字符seg并且index化,和label的index转化
        :param ex_index:
        :param example:
        :param label_list:
        :param max_seq_length:
        :param tokenizer:
        :return:
        """
        # 1. 构建label->id的映射
        label_map = {}
        if os.path.exists(os.path.join(self.output_dir, 'label2id.pkl')):
            with codecs.open(os.path.join(self.output_dir, 'label2id.pkl'), 'rb') as fd:
                label_map = pickle.load(fd)
        else:
            for i, label in enumerate(label_list):
                label_map[label] = i
            with codecs.open(os.path.join(self.output_dir, 'label2id.pkl'), 'wb') as fd:
                pickle.dump(label_map, fd)
        # 不考虑seq pair 分类的情况
        tokens_a = tokenizer.tokenize(example.text_a)

        # 截断，因为有句首和句尾的标识符
        if len(tokens_a) > max_seq_length - 2:
            tokens_a = tokens_a[0:(max_seq_length - 2)]

        tokens = []
        segment_ids = []
        tokens.append('[CLS]')
        segment_ids.append(0)
        for token in tokens_a:
            tokens.append(token)
            segment_ids.append(0)
        tokens.append('[SEP]')
        segment_ids.append(0)
        # 将字符转化为id形式
        input_ids = tokenizer.convert_tokens_to_ids(tokens)

        input_mask = [1] * len(input_ids)
        # 补全到max_seg_length
        while len(input_ids) < max_seq_length:
            input_ids.append(0)
            segment_ids.append(0)
            input_mask.append(0)
        if example.label is None:
            label_id = -1
        else:
            label_id = label_map[example.label]
        if ex_index < 2 and mode in ['train', 'dev']:
            tf.logging.info("*** Example ***")
            tf.logging.info("guid: %s" % (example.guid))
            tf.logging.info("tokens: %s" % " ".join(
                [tokenization.printable_text(x) for x in tokens]))
            tf.logging.info("input_ids: %s" % " ".join([str(x) for x in input_ids]))
            tf.logging.info("input_mask: %s" % " ".join([str(x) for x in input_mask]))
            tf.logging.info("segment_ids: %s" % " ".join([str(x) for x in segment_ids]))
            tf.logging.info("label: %s (id = %d)" % (example.label, label_id))

        feature = InputFeatures(input_ids=input_ids, input_mask=input_mask, segment_ids=segment_ids, label_id=label_id)
        return feature

    def file_based_convert_examples_to_features(self, examples, label_list, max_seq_length, tokenizer, output_file,
                                                mode):
        """
        将训练文件转化特征后，存储为tf_record格式，用于模型的读取
        :param mode:
        :param examples:
        :param label_list:
        :param max_seq_length:
        :param tokenizer:
        :param output_file:
        :return:
        """
        writer = tf.python_io.TFRecordWriter(path=output_file)
        # 将每一个样本转化为idx特征，封装到map中后进行序列化存储为record
        for ex_index, example in enumerate(examples):
            if ex_index % 10000 == 0:
                tf.logging.info("Writing example %d of %d" % (ex_index, len(examples)))
            feature = self.conver_single_example(ex_index, example, label_list, max_seq_length, tokenizer, mode)

            # 将输入数据转化为64位int 的list，这是必须的
            def create_int_feature(values):
                f = tf.train.Feature(int64_list=tf.train.Int64List(value=list(values)))
                return f

            features = collections.OrderedDict()
            features['input_ids'] = create_int_feature(feature.input_ids)
            features['input_mask'] = create_int_feature(feature.input_mask)
            features['segment_ids'] = create_int_feature(feature.segment_ids)
            features['label_ids'] = create_int_feature([feature.label_id])
            features["is_real_example"] = create_int_feature(
                [int(feature.is_real_example)])
            # 转化为Example 协议内存块
            tf_example = tf.train.Example(features=tf.train.Features(feature=features))
            writer.write(tf_example.SerializeToString())

    def file_based_input_fn_builder(self, input_file, seq_length, is_training,
                                    drop_remainder):
        """Creates an `input_fn` closure to be passed to TPUEstimator."""

        name_to_features = {
            "input_ids": tf.FixedLenFeature([seq_length], tf.int64),
            "input_mask": tf.FixedLenFeature([seq_length], tf.int64),
            "segment_ids": tf.FixedLenFeature([seq_length], tf.int64),
            "label_ids": tf.FixedLenFeature([], tf.int64),
            "is_real_example": tf.FixedLenFeature([], tf.int64),
        }

        def _decode_record(record, name_to_features):
            """Decodes a record to a TensorFlow example."""
            example = tf.parse_single_example(record, name_to_features)

            # tf.Example only supports tf.int64, but the TPU only supports tf.int32.
            # So cast all int64 to int32.
            for name in list(example.keys()):
                t = example[name]
                if t.dtype == tf.int64:
                    t = tf.to_int32(t)
                example[name] = t

            return example

        def input_fn(params):
            """The actual input function."""
            batch_size = params["batch_size"]

            # For training, we want a lot of parallel reading and shuffling.
            # For eval, we want no shuffling and parallel reading doesn't matter.
            d = tf.data.TFRecordDataset(input_file)
            if is_training:
                d = d.repeat()
                d = d.shuffle(buffer_size=100)

            d = d.apply(
                tf.contrib.data.map_and_batch(
                    lambda record: _decode_record(record, name_to_features),
                    batch_size=batch_size,
                    drop_remainder=drop_remainder))

            return d

        return input_fn

    def create_model(self, is_training, input_ids, input_mask, segment_ids,
                     labels, num_labels):
        """Creates a classification model."""
        model = modeling.BertModel(
            config=self.bert_config,
            is_training=is_training,
            input_ids=input_ids,
            input_mask=input_mask,
            token_type_ids=segment_ids)

        output_layer = model.get_pooled_output()

        hidden_size = output_layer.shape[-1].value

        output_weights = tf.get_variable(
            "output_weights", [num_labels, hidden_size],
            initializer=tf.truncated_normal_initializer(stddev=0.02))

        output_bias = tf.get_variable(
            "output_bias", [num_labels], initializer=tf.zeros_initializer())

        with tf.variable_scope("loss"):
            if is_training:
                # I.e., 0.1 dropout
                output_layer = tf.nn.dropout(output_layer, keep_prob=0.9)

            logits = tf.matmul(output_layer, output_weights, transpose_b=True)
            logits = tf.nn.bias_add(logits, output_bias)
            probabilities = tf.nn.softmax(logits, axis=-1)
            log_probs = tf.nn.log_softmax(logits, axis=-1)

            one_hot_labels = tf.one_hot(labels, depth=num_labels, dtype=tf.float32)

            per_example_loss = -tf.reduce_sum(one_hot_labels * log_probs, axis=-1)
            loss = tf.reduce_mean(per_example_loss)

            return loss, per_example_loss, logits, probabilities

    def model_fn_builder(self, num_labels, init_checkpoint, learning_rate,
                         num_train_steps, num_warmup_steps):

        def model_fn(features, labels, mode, params):  # pylint: disable=unused-argument
            """The `model_fn` for TPUEstimator."""

            tf.logging.info("*** Features ***")
            for name in sorted(features.keys()):
                tf.logging.info("  name = %s, shape = %s" % (name, features[name].shape))

            input_ids = features["input_ids"]
            input_mask = features["input_mask"]
            segment_ids = features["segment_ids"]
            label_ids = features["label_ids"]
            if "is_real_example" in features:
                is_real_example = tf.cast(features["is_real_example"], dtype=tf.float32)
            else:
                is_real_example = tf.ones(tf.shape(label_ids), dtype=tf.float32)

            is_training = (mode == tf.estimator.ModeKeys.TRAIN)

            total_loss, per_example_loss, logits, probabilities = self.create_model(
                is_training, input_ids, input_mask, segment_ids, label_ids,
                num_labels)

            tvars = tf.trainable_variables()
            initialized_variable_names = {}
            if init_checkpoint:
                assignment_map, initialized_variable_names = modeling.get_assignment_map_from_checkpoint(tvars,
                                                                                                         init_checkpoint)
                tf.train.init_from_checkpoint(init_checkpoint, assignment_map)
            tf.logging.info("**** Trainable Variables ****")
            for var in tvars:
                init_string = ""
                if var.name in initialized_variable_names:
                    init_string = ", *INIT_FROM_CKPT*"
                tf.logging.info("  name = %s, shape = %s%s", var.name, var.shape,
                                init_string)
            output_spec = None
            if mode == tf.estimator.ModeKeys.TRAIN:
                train_op = optimization.create_optimizer(
                    total_loss, learning_rate, num_train_steps, num_warmup_steps, use_tpu=False)

                output_spec = tf.estimator.EstimatorSpec(
                    mode=mode,
                    loss=total_loss,
                    train_op=train_op)
            elif mode == tf.estimator.ModeKeys.EVAL:
                def metric_fn(per_example_loss, label_ids, logits, is_real_example):
                    predictions = tf.argmax(logits, axis=-1, output_type=tf.int32)
                    accuracy = tf.metrics.accuracy(
                        labels=label_ids, predictions=predictions, weights=is_real_example)
                    loss = tf.metrics.mean(values=per_example_loss, weights=is_real_example)
                    return {
                        "eval_accuracy": accuracy,
                        "eval_loss": loss,
                    }

                eval_metrics = metric_fn(per_example_loss, label_ids, logits, is_real_example)
                output_spec = tf.estimator.EstimatorSpec(
                    mode=mode,
                    loss=total_loss,
                    eval_metric_ops=eval_metrics)
            else:
                raise ValueError("mod should be one of `train`, `dev`.")
            return output_spec

        return model_fn

    def run(self, output_dir, save_summary_steps, save_checkpoints_steps, data_dir, train_batch_size, learning_rate,
            num_train_epochs, warmup_proportion, max_seq_length, mod, eval_batch_size=4):
        self.output_dir = output_dir
        tf.gfile.MakeDirs(output_dir)
        run_config = tf.estimator.RunConfig(
            model_dir=output_dir,
            save_summary_steps=save_summary_steps,
            save_checkpoints_steps=save_checkpoints_steps,
            keep_checkpoint_max=5,
            log_step_count_steps=32,
            session_config=tf.ConfigProto(log_device_placement=True)
            # session_config=tf.ConfigProto(log_device_placement=True,
            #                               device_count={'GPU': 1}))
        )
        data_processor = MyProcessor()
        label_list = data_processor.get_labels()
        tf.logging.info('************ label_list=' + ' '.join(label_list))
        if mod == "train":
            train_examples = data_processor.get_train_examples(data_dir=data_dir)
            num_train_steps = int(
                len(train_examples) / train_batch_size * num_train_epochs)
            num_warmup_steps = int(num_train_steps * warmup_proportion)
        elif mod == "dev":
            train_examples = None
            num_train_steps = None
            num_warmup_steps = None
        else:
            raise ValueError("mod must be one of `train`, `dev`.")
        model_fn = self.model_fn_builder(
            num_labels=len(label_list),
            init_checkpoint=self.init_checkpoint,
            learning_rate=learning_rate,
            num_train_steps=num_train_steps,
            num_warmup_steps=num_warmup_steps
        )
        params = {
            'batch_size': train_batch_size,
        }
        estimator = tf.estimator.Estimator(
            model_fn=model_fn,
            config=run_config,
            params=params,
        )
        if mod == "train":
            self.train(output_dir, train_examples, label_list, max_seq_length, train_batch_size, num_train_steps,
                       estimator)
        elif mod == "dev":
            eval_examples = data_processor.get_dev_examples(data_dir=data_dir)
            self.eval(output_dir, eval_examples, label_list, max_seq_length, eval_batch_size, estimator)
        else:
            raise ValueError("mod should be one of `train`, `dev`.")

    def train(self, output_dir, train_examples, label_list, max_seq_length, train_batch_size, num_train_steps,
              estimator,
              ):
        train_file = os.path.join(output_dir, "train.tf_record")
        self.file_based_convert_examples_to_features(
            train_examples, label_list, max_seq_length, self.tokenizer, train_file, 'train')
        tf.logging.info("***** Running training *****")
        tf.logging.info("  Num examples = %d", len(train_examples))
        tf.logging.info("  Batch size = %d", train_batch_size)
        tf.logging.info("  Num steps = %d", num_train_steps)
        train_input_fn = self.file_based_input_fn_builder(
            input_file=train_file,
            seq_length=max_seq_length,
            is_training=True,
            drop_remainder=True)
        estimator.train(input_fn=train_input_fn, max_steps=num_train_steps)

    def eval(self, output_dir, eval_examples, label_list, max_seq_length, eval_batch_size, estimator):
        num_actual_eval_examples = len(eval_examples)
        eval_file = os.path.join(output_dir, "eval.tf_record")
        self.file_based_convert_examples_to_features(
            eval_examples, label_list, max_seq_length, self.tokenizer, eval_file, mode='dev')
        tf.logging.info("***** Running evaluation *****")
        tf.logging.info("  Num examples = %d (%d actual, %d padding)",
                        len(eval_examples), num_actual_eval_examples,
                        len(eval_examples) - num_actual_eval_examples)
        tf.logging.info("  Batch size = %d", eval_batch_size)
        eval_steps = None
        eval_drop_remainder = False
        eval_input_fn = self.file_based_input_fn_builder(
            input_file=eval_file,
            seq_length=max_seq_length,
            is_training=False,
            drop_remainder=eval_drop_remainder)

        result = estimator.evaluate(input_fn=eval_input_fn, steps=eval_steps)
        output_eval_file = os.path.join(output_dir, "eval_results.txt")
        with tf.gfile.GFile(output_eval_file, "w") as writer:
            tf.logging.info("***** Eval results *****")
            for key in sorted(result.keys()):
                tf.logging.info("  %s = %s", key, str(result[key]))
                writer.write("%s = %s\n" % (key, str(result[key])))


if __name__ == '__main__':
    use_model = BertClassifier()
    use_model.run(output_dir="output_dir", save_summary_steps=10, save_checkpoints_steps=10, data_dir="../task",
                  train_batch_size=16, learning_rate=1e-5, num_train_epochs=5, max_seq_length=32, warmup_proportion=0.1,
                  mod="train", eval_batch_size=8)



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: __init__.py.py
@time: 2020/3/24 11:11

这一行开始写关于本文件的说明与解释


"""
# Make sure that leslie is running on Python 3.6.1 or later
# (to avoid running into this bug: https://bugs.python.org/issue29246)
import sys

if sys.version_info < (3, 6, 1):
    raise RuntimeError("Leslie requires Python 3.6.1 or later")

# We get a lot of these spurious warnings,
# see https://github.com/ContinuumIO/anaconda-issues/issues/6678
import warnings  # noqa

warnings.filterwarnings("ignore", message="numpy.dtype size changed")
warnings.filterwarnings("ignore", message="numpy.ufunc size changed")

try:
    # On some systems this prevents the dreaded
    # ImportError: dlopen: cannot load any more object with static TLS
    import tensorflow, numpy  # noqa

except ModuleNotFoundError:
    print(
        "Using Leslie requires the python packages, "
        "tensorflow and Numpy to be installed."
    )
    raise

from leslie.version import VERSION as __version__  # noqa


# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com
@version: 1.0

@file: __main__.py 
@time: 2020/6/13 下午3:28

这一行开始写关于本文件的说明与解释

"""
import logging
import os
import sys

LEVEL = logging.DEBUG

sys.path.insert(0, os.path.dirname(os.path.abspath(os.path.join(__file__, os.pardir))))
logging.basicConfig(format="%(asctime)s - %(levelname)s - %(name)s - %(message)s", level=LEVEL)

from leslie.commands import main  # noqa


def run():
    main(program="Text")


if __name__ == "__main__":
    run()



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: __init__.py
@time: 2020/5/3 20:19

这一行开始写关于本文件的说明与解释


"""
from leslie.data.dataset_readers.dataset_reader import DatasetReader
# from leslie.data.dataloader import DataLoader
from leslie.data.vocabulary import Vocabulary



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: vocabulary.py
@time: 2019/12/19 9:48

这一行开始写关于本文件的说明与解释

A Vocabulary maps strings to integers, allowing for strings to be mapped to an
out-of-vocabulary token.
"""
import codecs
import copy
import logging
import os
from collections import defaultdict
from typing import Callable, Dict, Iterable, List, Optional, Set, Union
from leslie.common.params import Params
from tqdm import tqdm

from leslie.common.registrable import Registrable
from leslie.common.checks import ConfigurationError
from leslie.common.util import namespace_match

DEFAULT_NON_PADDED_NAMESPACES = ("*tags", "*labels")
DEFAULT_PADDING_TOKEN = "@@PADDING@@"
DEFAULT_OOV_TOKEN = "@@UNKNOWN@@"
NAMESPACE_PADDING_FILE = 'non_padded_namespaces.txt'

logger = logging.getLogger(__name__)


class _NamespaceDependentDefaultDict(defaultdict):
    def __init__(self,
                 non_padded_namespaces: Iterable[str],
                 padded_function: Callable[[], any],
                 non_padded_function: Callable[[], any]) -> None:
        self._non_padded_namespaces = set(non_padded_namespaces)
        self._padded_function = padded_function
        self._non_padded_function = non_padded_function
        super(_NamespaceDependentDefaultDict, self).__init__()

    def __missing__(self, key: str):
        if any(namespace_match(pattern, key) for pattern in self._non_padded_namespaces):
            value = self._non_padded_function()
        else:
            value = self._padded_function()
        dict.__setitem__(self, key, value)
        return value

    def add_non_padded_namespaces(self, non_padded_namespaces: Set[str]):
        # add non_padded_namespaces which weren't already present
        self._non_padded_namespaces.update(non_padded_namespaces)


class _TokenToIndexDefaultDict(_NamespaceDependentDefaultDict):
    def __init__(self, non_padded_namespaces: Set[str], padding_token: str, oov_token: str) -> None:
        super(_TokenToIndexDefaultDict, self).__init__(
            non_padded_namespaces, lambda: {padding_token: 0, oov_token: 1}, lambda: {}
        )


class _IndexToTokenDefaultDict(_NamespaceDependentDefaultDict):
    def __init__(self, non_padded_namespaces: Set[str], padding_token: str, oov_token: str) -> None:
        super(_IndexToTokenDefaultDict, self).__init__(
            non_padded_namespaces, lambda: {0: padding_token, 1: oov_token}, lambda: {}
        )


def _read_pretrained_tokens(embeddings_file_uri: str) -> List[str]:
    # todo
    return []


def pop_max_vocab_size(params: Params) -> Union[int, Dict[str, int]]:
    """
    max_vocab_size limits the size of the vocabulary, not including the @@UNKNOWN@@ token.
    max_vocab_size is allowed to be either an int or a Dict[str, int] (or nothing).
    But it could also be a string representing an int (in the case of environment variable
    substitution). So we need some complex logic to handle it.
    """
    size = params.pop("max_vocab_size", None, keep_as_dict=True)

    if isinstance(size, dict):
        # This is the Dict[str, int] case.
        return size
    elif size is not None:
        # This is the int / str case.
        return int(size)
    else:
        return None


class Vocabulary(Registrable):
    def __init__(self,
                 counter: Dict[str, Dict[str, int]] = None,
                 min_count: Dict[str, int] = None,
                 max_vocab_size: Union[int, Dict[str, int]] = None,
                 non_padded_namespaces: Iterable[str] = DEFAULT_NON_PADDED_NAMESPACES,
                 pretrained_files: Optional[Dict[str, str]] = None,
                 only_include_pretrained_words: bool = False,
                 tokens_to_add: Dict[str, List[str]] = None,
                 min_pretrained_embeddings: Dict[str, int] = None,
                 padding_token: Optional[str] = DEFAULT_PADDING_TOKEN,
                 oov_token: Optional[str] = DEFAULT_OOV_TOKEN,
                 ) -> None:
        self._padding_token = padding_token if padding_token is not None else DEFAULT_PADDING_TOKEN
        self._oov_token = oov_token if oov_token is not None else DEFAULT_OOV_TOKEN

        self._non_padded_namespaces = set(non_padded_namespaces)

        self._token_to_index = _TokenToIndexDefaultDict(
            self._non_padded_namespaces, self._padding_token, self._oov_token
        )
        self._index_to_token = _IndexToTokenDefaultDict(
            self._non_padded_namespaces, self._padding_token, self._oov_token
        )

        self._retained_counter: Optional[Dict[str, Dict[str, int]]] = None
        # Made an empty vocabulary, now extend it.
        self._extend(
            counter,
            min_count,
            max_vocab_size,
            non_padded_namespaces,
            pretrained_files,
            only_include_pretrained_words,
            tokens_to_add,
            min_pretrained_embeddings,
        )

    def __getstate__(self):
        """
        Need to sanitize default_dict and default_like objects
        by converting them to vanilla dicts when we pickle the vocabulary.
        """
        state = copy.copy(self.__dict__)
        state["_token_to_idx"] = dict(state["_token_to_idx"])

    def __setstate__(self, state):
        """
        Conversely, when we unpickle, we need to reload the plain dicts
        into our special DefaultDict subclasses.
        """

        self.__dict__ = copy.copy(state)
        self._token_to_index = _TokenToIndexDefaultDict(
            self._non_padded_namespaces, self._padding_token, self._oov_token
        )
        self._token_to_index.update(state["_token_to_index"])
        self._index_to_token = _IndexToTokenDefaultDict(
            self._non_padded_namespaces, self._padding_token, self._oov_token
        )
        self._index_to_token.update(state["_index_to_token"])

    def _extend(
            self,
            counter: Dict[str, Dict[str, int]] = None,
            min_count: Dict[str, int] = None,
            max_vocab_size: Union[int, Dict[str, int]] = None,
            non_padded_namespaces: Iterable[str] = DEFAULT_NON_PADDED_NAMESPACES,
            pretrained_files: Optional[Dict[str, str]] = None,
            only_include_pretrained_words: bool = False,
            tokens_to_add: Dict[str, List[str]] = None,
            min_pretrained_embeddings: Dict[str, int] = None,
    ) -> None:
        if not isinstance(max_vocab_size, dict):
            int_max_vocab_size = max_vocab_size
            max_vocab_size = defaultdict(lambda: int_max_vocab_size)  # type: ignore
        min_count = min_count or {}
        pretrained_files = pretrained_files or {}
        min_pretrained_embeddings = min_pretrained_embeddings or {}
        non_padded_namespaces = set(non_padded_namespaces)
        counter = counter or {}
        tokens_to_add = tokens_to_add or {}

        self._retained_counter = counter
        # Make sure vocabulary extension is safe.
        current_namespaces = {*self._token_to_index}
        extension_namespaces = {*counter, *tokens_to_add}

        for namespace in current_namespaces & extension_namespaces:
            # if new namespace was already present
            # Either both should be padded or none should be.
            original_padded = not any(
                namespace_match(pattern, namespace) for pattern in self._non_padded_namespaces
            )
            extension_padded = not any(
                namespace_match(pattern, namespace) for pattern in non_padded_namespaces
            )
            if original_padded != extension_padded:
                raise ConfigurationError(
                    "Common namespace {} has conflicting ".format(namespace)
                    + "setting of padded = True/False. "
                    + "Hence extension cannot be done."
                )

        # Add new non-padded namespaces for extension
        self._token_to_index.add_non_padded_namespaces(non_padded_namespaces)
        self._index_to_token.add_non_padded_namespaces(non_padded_namespaces)
        self._non_padded_namespaces.update(non_padded_namespaces)

        for namespace in counter:
            if namespace in pretrained_files:
                pretrained_list = _read_pretrained_tokens(pretrained_files[namespace])
                min_embeddings = min_pretrained_embeddings.get(namespace, 0)
                if min_embeddings > 0:
                    tokens_old = tokens_to_add.get(namespace, [])
                    tokens_new = pretrained_list[:min_embeddings]
                    tokens_to_add[namespace] = tokens_old + tokens_new
                pretrained_set = set(pretrained_list)
            else:
                pretrained_set = None
            token_counts = list(counter[namespace].items())
            token_counts.sort(key=lambda x: x[1], reverse=True)
            try:
                max_vocab = max_vocab_size[namespace]
            except KeyError:
                max_vocab = None
            if max_vocab:
                token_counts = token_counts[:max_vocab]
            for token, count in token_counts:
                if pretrained_set is not None:
                    if only_include_pretrained_words:
                        if token in pretrained_set and count >= min_count.get(namespace, 1):
                            self.add_token_to_namespace(token, namespace)
                    elif token in pretrained_set or count >= min_count.get(namespace, 1):
                        self.add_token_to_namespace(token, namespace)
                elif count >= min_count.get(namespace, 1):
                    self.add_token_to_namespace(token, namespace)

        for namespace, tokens in tokens_to_add.items():
            for token in tokens:
                self.add_token_to_namespace(token, namespace)

    def add_token_to_namespace(self, token: str, namespace: str = "tokens") -> int:
        """
        Adds ``token`` to the index, if it is not already present.  Either way, we return the index of
        the token.
        """
        if not isinstance(token, str):
            raise ValueError(
                "Vocabulary tokens must be strings, or saving and loading will break."
                "  Got %s (with type %s)" % (repr(token), type(token))
            )
        if token not in self._token_to_index[namespace]:
            index = len(self._token_to_index[namespace])
            self._token_to_index[namespace][token] = index
            self._index_to_token[namespace][index] = token
            return index
        else:
            return self._token_to_index[namespace][token]

    def get_token_index(self, token: str, namespace: str = "tokens") -> int:
        if token in self._token_to_index[namespace]:
            return self._token_to_index[namespace][token]
        else:
            try:
                return self._token_to_index[namespace][self._oov_token]
            except KeyError:
                raise

    def get_token_from_index(self, index: int, namespace: str = "tokens") -> str:
        return self._index_to_token[namespace][index]

    @classmethod
    def from_instances(
            cls,
            instances: Iterable["adi.Instance"],
            min_count: Dict[str, int] = None,
            max_vocab_size: Union[int, Dict[str, int]] = None,
            non_padded_namespaces: Iterable[str] = DEFAULT_NON_PADDED_NAMESPACES,
            pretrained_files: Optional[Dict[str, str]] = None,
            only_include_pretrained_words: bool = False,
            tokens_to_add: Dict[str, List[str]] = None,
            min_pretrained_embeddings: Dict[str, int] = None,
            padding_token: Optional[str] = DEFAULT_PADDING_TOKEN,
            oov_token: Optional[str] = DEFAULT_OOV_TOKEN,
    ) -> "Vocabulary":
        """
        Constructs a vocabulary given a collection of `Instances` and some parameters.
        We count all of the vocabulary items in the instances, then pass those counts
        and the other parameters, to :func:`__init__`.  See that method for a description
        of what the other parameters do.
        """
        padding_token = padding_token if padding_token is not None else DEFAULT_PADDING_TOKEN
        oov_token = oov_token if oov_token is not None else DEFAULT_OOV_TOKEN
        namespace_token_counts: Dict[str, Dict[str, int]] = defaultdict(lambda: defaultdict(int))
        for instance in tqdm(instances):
            instance.count_vocab_items(namespace_token_counts)

        return cls(
            counter=namespace_token_counts,
            min_count=min_count,
            max_vocab_size=max_vocab_size,
            non_padded_namespaces=non_padded_namespaces,
            pretrained_files=pretrained_files,
            only_include_pretrained_words=only_include_pretrained_words,
            tokens_to_add=tokens_to_add,
            min_pretrained_embeddings=min_pretrained_embeddings,
            padding_token=padding_token,
            oov_token=oov_token,
        )

    def get_vocab_size(self, namespace: str = "tokens") -> int:
        return len(self._token_to_index[namespace])

    @classmethod
    def from_files(
            cls,
            directory: str,
            padding_token: Optional[str] = DEFAULT_PADDING_TOKEN,
            oov_token: Optional[str] = DEFAULT_OOV_TOKEN,
    ) -> "Vocabulary":
        """
        Loads a ``Vocabulary`` that was serialized using ``save_to_files``.
        Parameters
        ----------
        directory : ``str``
            The directory containing the serialized vocabulary.
            :param directory:
            :param padding_token:
            :param oov_token:
        """
        logger.info("Loading token dictionary from %s.", directory)
        padding_token = padding_token if padding_token is not None else DEFAULT_PADDING_TOKEN
        oov_token = oov_token if oov_token is not None else DEFAULT_OOV_TOKEN
        vocab = cls(
            padding_token=padding_token,
            oov_token=oov_token,
        )

        # Check every file in the directory.
        for namespace_filename in os.listdir(directory):
            if namespace_filename.startswith("."):
                continue
            namespace = namespace_filename.replace(".txt", "")
            if any(namespace_match(pattern, namespace) for pattern in DEFAULT_NON_PADDED_NAMESPACES):
                is_padded = False
            else:
                is_padded = True
            filename = os.path.join(directory, namespace_filename)
            vocab.set_from_file(filename, is_padded, namespace=namespace, oov_token=oov_token)

        return vocab

    def set_from_file(
            self,
            filename: str,
            is_padded: bool = True,
            oov_token: str = DEFAULT_OOV_TOKEN,
            namespace: str = "tokens",
    ):
        """
        If you already have a vocabulary file for a trained model somewhere, and you really want to
        use that vocabulary file instead of just setting the vocabulary from a dataset, for
        whatever reason, you can do that with this method.  You must specify the namespace to use,
        and we assume that you want to use padding and OOV tokens for this.
        Parameters
        ----------
        filename : ``str``
            The file containing the vocabulary to load.  It should be formatted as one token per
            line, with nothing else in the line.  The index we assign to the token is the line
            number in the file (1-indexed if ``is_padded``, 0-indexed otherwise).  Note that this
            file should contain the OOV token string!
        is_padded : ``bool``, optional (default=True)
            Is this vocabulary padded?  For token / word / character vocabularies, this should be
            ``True``; while for tag or label vocabularies, this should typically be ``False``.  If
            ``True``, we add a padding token with index 0, and we enforce that the ``oov_token`` is
            present in the file.
        oov_token : ``str``, optional (default=DEFAULT_OOV_TOKEN)
            What token does this vocabulary use to represent out-of-vocabulary characters?  This
            must show up as a line in the vocabulary file.  When we find it, we replace
            ``oov_token`` with ``self._oov_token``, because we only use one OOV token across
            namespaces.
        namespace : ``str``, optional (default="tokens")
            What namespace should we overwrite with this vocab file?
        """
        # if is_padded:
        #     self._token_to_index[namespace] = {self._padding_token: 0}
        #     self._index_to_token[namespace] = {0: self._padding_token}
        # else:
        #     self._token_to_index[namespace] = {}
        #     self._index_to_token[namespace] = {}
        #  取消is_pad判断
        self._token_to_index[namespace] = {}
        self._index_to_token[namespace] = {}

        with codecs.open(filename, "r", "utf-8") as input_file:
            lines = input_file.read().split("\n")
            # Be flexible about having final newline or not
            if lines and lines[-1] == "":
                lines = lines[:-1]
            for i, line in enumerate(lines):
                index = i + 1 if is_padded else i
                token = line.replace("@@NEWLINE@@", "\n")
                if token == oov_token:
                    token = self._oov_token
                self._token_to_index[namespace][token] = index
                self._index_to_token[namespace][index] = token
        # if is_padded:
        assert self._oov_token in self._token_to_index[namespace], "OOV token not found!"

    @classmethod
    def from_params(cls, params: Params, instances: Iterable["adi.Instance"] = None):
        """
        There are two possible ways to build a vocabulary; from a
        collection of instances, using :func:`Vocabulary.from_instances`, or
        from a pre-saved vocabulary, using :func:`Vocabulary.from_files`.
        You can also extend pre-saved vocabulary with collection of instances
        using this method. This method wraps these options, allowing their
        specification from a ``Params`` object, generated from a JSON
        configuration file.
        Parameters
        ----------
        params: Params, required.
        instances: Iterable['adi.Instance'], optional
            If ``params`` doesn't contain a ``directory_path`` key,
            the ``Vocabulary`` can be built directly from a collection of
            instances (i.e. a dataset). If ``extend`` key is set False,
            dataset instances will be ignored and final vocabulary will be
            one loaded from ``directory_path``. If ``extend`` key is set True,
            dataset instances will be used to extend the vocabulary loaded
            from ``directory_path`` and that will be final vocabulary used.
        Returns
        -------
        A ``Vocabulary``.
        """

        # Vocabulary is ``Registrable`` so that you can configure a custom subclass,
        # but (unlike most of our registrables) almost everyone will want to use the
        # base implementation. So instead of having an abstract ``VocabularyBase`` or
        # such, we just add the logic for instantiating a registered subclass here,
        # so that most users can continue doing what they were doing.
        vocab_type = params.pop("type", None)
        if vocab_type is not None:
            return cls.by_name(vocab_type).from_params(params=params, instances=instances)

        extend = params.pop("extend", False)
        vocabulary_directory = params.pop("directory_path", None)
        if not vocabulary_directory and not instances:
            raise ConfigurationError(
                "You must provide either a Params object containing a "
                "vocab_directory key or a Dataset to build a vocabulary from."
            )
        if extend and not instances:
            raise ConfigurationError(
                "'extend' is true but there are not instances passed to extend."
            )
        if extend and not vocabulary_directory:
            raise ConfigurationError(
                "'extend' is true but there is not 'directory_path' to extend from."
            )

        if vocabulary_directory and instances:
            if extend:
                logger.info("Loading Vocab from files and extending it with dataset.")
            else:
                logger.info("Loading Vocab from files instead of dataset.")

        padding_token = params.pop("padding_token", DEFAULT_PADDING_TOKEN)
        oov_token = params.pop("oov_token", DEFAULT_OOV_TOKEN)

        if vocabulary_directory:
            vocab = cls.from_files(vocabulary_directory, padding_token, oov_token)
            if not extend:
                params.assert_empty("Vocabulary - from files")
                return vocab
        if extend:
            vocab.extend_from_instances(params, instances=instances)
            return vocab
        min_count = params.pop("min_count", None, keep_as_dict=True)
        max_vocab_size = pop_max_vocab_size(params)
        non_padded_namespaces = params.pop("non_padded_namespaces", DEFAULT_NON_PADDED_NAMESPACES)
        pretrained_files = params.pop("pretrained_files", {}, keep_as_dict=True)
        min_pretrained_embeddings = params.pop("min_pretrained_embeddings", None)
        only_include_pretrained_words = params.pop_bool("only_include_pretrained_words", False)
        tokens_to_add = params.pop("tokens_to_add", None)
        params.assert_empty("Vocabulary - from dataset")
        return cls.from_instances(
            instances=instances,
            min_count=min_count,
            max_vocab_size=max_vocab_size,
            non_padded_namespaces=non_padded_namespaces,
            pretrained_files=pretrained_files,
            only_include_pretrained_words=only_include_pretrained_words,
            tokens_to_add=tokens_to_add,
            min_pretrained_embeddings=min_pretrained_embeddings,
            padding_token=padding_token,
            oov_token=oov_token,
        )

    def save_to_files(self, directory: str) -> None:
        """
        Persist this Vocabulary to files so it can be reloaded later.
        Each namespace corresponds to one file.
        Parameters
        ----------
        directory : ``str``
            The directory where we save the serialized vocabulary.
        """
        os.makedirs(directory, exist_ok=True)
        if os.listdir(directory):
            logging.warning("vocabulary serialization directory %s is not empty", directory)

        with codecs.open(os.path.join(directory, NAMESPACE_PADDING_FILE), 'w', 'utf-8') as namespace_file:
            for namespace_str in self._non_padded_namespaces:
                print(namespace_str, file=namespace_file)

        for namespace, mapping in self._index_to_token.items():
            # Each namespace gets written to its own file, in index order.
            with codecs.open(os.path.join(directory, namespace + '.txt'), 'w', 'utf-8') as token_file:
                num_tokens = len(mapping)
                # start_index = 1 if mapping[0] == self._padding_token else 0    ## 这里取消对is_pad的判断
                for i in range(0, num_tokens):
                    print(mapping[i].replace('\n', '@@NEWLINE@@'), file=token_file)



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: simple_text_classifier.py
@time: 2019/12/21 10:46

这一行开始写关于本文件的说明与解释


"""
import tensorflow as tf

from leslie.models.model import Model


class AverPoolingTextClassifierModel(Model):
    def __init__(self, vocab_size, embed_size):
        super(AverPoolingTextClassifierModel, self).__init__()
        self.embedding = tf.keras.layers.Embedding(vocab_size, embed_size, trainable=True)
        self.average = tf.keras.layers.GlobalAveragePooling1D()
        self.d1 = tf.keras.layers.Dense(64, activation='relu')
        self.d2 = tf.keras.layers.Dense(4, activation='softmax')

    def call(self, inputs, training=None, mask=None):
        x = self.embedding(inputs)
        x = self.average(x)
        x = self.d1(x)
        x = self.d2(x)
        return x



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com
@version: 1.0

@file: version.py 
@time: 2020/6/13 下午3:10

这一行开始写关于本文件的说明与解释

"""
import os

_MAJOR = "0"
_MINOR = "0"
# On master and in a nightly release the patch should be one ahead of the last
# released build.
_PATCH = "1rc1"
# See https://semver.org/#is-v123-a-semantic-version for the semantics.

VERSION_SHORT = "{0}.{1}".format(_MAJOR, _MINOR)
VERSION = "{0}.{1}.{2}".format(_MAJOR, _MINOR, _PATCH)



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: text_classifier_dataset_reader.py
@time: 2019/11/25 16:05

这一行开始写关于本文件的说明与解释


"""
from typing import Iterable, Dict, Union
import json

from leslie.data.instance import Instance
from leslie.data.fields.field import Field
from leslie.data.fields.text_field import TextField
from leslie.data.fields.label_field import LabelField
from leslie.data.token_indexers.token_indexer import TokenIndexer
from leslie.data.tokenizers.tokenizer import Tokenizer
from leslie.data.dataset_readers.dataset_reader import DatasetReader


class TextClassifierDatasetReader(DatasetReader):
    """
        Reads tokens and their labels from a labeled leslie classification dataset.
        Expects a "leslie" field and a "label" field in JSON format.
        The output of ``read`` is a list of ``Instance`` s with the fields:
            tokens: ``TextField`` and
            label: ``LabelField``
        Parameters
        ----------
        token_indexers : ``Dict[str, TokenIndexer]``, optional
            optional (default=``{"tokens": SingleIdTokenIndexer()}``)
            We use this to define the input representation for the leslie.
            See :class:`TokenIndexer`.
        tokenizer : ``Tokenizer``, optional (default = ``{"tokens": CharacterTokenizer()}``)
            Tokenizer to use to split the input leslie into words or other kinds of tokens.
        max_sequence_length: ``int``, optional (default = ``None``)
            If specified, will truncate tokens to specified maximum length.
        skip_label_indexing: ``bool``, optional (default = ``False``)
            Whether or not to skip label indexing. You might want to skip label indexing if your
            labels are numbers, so the dataset reader doesn't re-number them starting from 0.
        """

    def __init__(self,
                 token_indexers: Dict[str, TokenIndexer],
                 tokenizer: Tokenizer = None,
                 max_sequence_length: int = None,
                 skip_label_indexing: bool = False) -> None:
        self._tokenizer = tokenizer
        self._token_indexers = token_indexers
        self._max_sequence_length = max_sequence_length
        self._skip_label_indexing = skip_label_indexing

    def _read(self, file_path: str) -> Iterable:
        with open(file_path, "r") as data_file:
            for line in data_file:
                line_data = json.loads(line)
                text = line_data["query"]
                label = line_data["label"]
                instance = self.text_to_instance(text=text, label=label)
                if instance:
                    yield instance

    def text_to_instance(self, text: str, label: Union[str, int]) -> Instance:
        fields: Dict[str, Field] = {}
        tokens = self._tokenizer.tokenize(text)
        if self._max_sequence_length:
            tokens = self._truncated(tokens)
        fields["tokens"] = TextField(tokens, self._token_indexers)
        fields["label"] = LabelField(label, skip_indexing=self._skip_label_indexing)
        return Instance(fields)

    def _truncated(self, tokens):
        if len(tokens) > self._max_sequence_length:
            tokens = tokens[:self._max_sequence_length]
        return tokens



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: intent_slot_model.py
@time: 2020/3/24 11:13

这一行开始写关于本文件的说明与解释


"""
import tensorflow as tf
import numpy as np

from leslie.models.model import Model


class IntentSlotModel(Model):
    def __init__(self):
        super(IntentSlotModel).__init__()

    def call(self, inputs, training=None, mask=None):
        raise NotImplementedError

    def train(self):
        pass



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: dataset_reader.py
@time: 2019/12/18 11:39

这一行开始写关于本文件的说明与解释


"""
from typing import Iterable, Iterator, Callable, List

from leslie.data.instance import Instance
from leslie.common.registrable import Registrable
from leslie.common.checks import ConfigurationError


class DatasetReader(Registrable):
    """
    dataset_reader basic
    """

    def read(self, file_path: str) -> List:
        instances = self._read(file_path=file_path)
        if not isinstance(instances, list):
            instances = [instance for instance in instances]
        if not instances:
            raise ConfigurationError("No instances were read from the given filepath {}. "
                                     "Is the path correct?".format(file_path))
        return instances

    def _read(self, file_path: str) -> Iterable:
        raise NotImplementedError

    def text_to_instance(self, *inputs) -> Instance:
        raise NotImplementedError



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: __init__.py
@time: 2019/12/11 14:48

这一行开始写关于本文件的说明与解释


"""
from leslie.common.registrable import Registrable
from leslie.common.params import Params
from leslie.common.lazy import Lazy



# _*_ coding:utf-8 _*_
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file cnn_test_classifier.py
@time  22:17
这一行开始写文本解释与说明

"""
import tensorflow as tf

from leslie.models.model import Model


class CnnModel(Model):
    def __init__(self):
        super(CnnModel, self).__init__()
        self.embedding = tf.keras.layers.Embedding(3000, 512)
        self.conv_1 = tf.keras.layers.Conv1D(16, kernel_size=5, name="conv_1", activation="relu")
        # self.pool = tf.keras.layers.MaxPool1D()
        # self.conv_2 = tf.keras.layers.Conv1D(128, kernel_size=2, name="conv_2", activation="relu")
        self.flatten = tf.keras.layers.GlobalAveragePooling1D()
        self.dense = tf.keras.layers.Dense(4, activation='softmax')

    def call(self, inputs, training=None, mask=None):
        x = self.embedding(inputs)
        x = self.conv_1(x)
        # x = self.pool(x)
        # x = self.conv_2(x)
        # x = self.pool(x)
        x = self.flatten(x)
        x = self.dense(x)
        return x



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com
@version: 1.0

@file: __init__.py 
@time: 2020/6/23 下午11:55

这一行开始写关于本文件的说明与解释

"""


def func():
    pass


if __name__ == '__main__':
    func()



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: text_classifier_model.py
@time: 2019/11/25 16:24

这一行开始写关于本文件的说明与解释


"""
import tensorflow as tf
import numpy as np


class TextClassifierModel(object):
    """
    文本分类模型基类
    """

    def __init__(self,
                 training_data,
                 training_labels,
                 val_data,
                 val_labels,
                 word_index,
                 model_path,
                 max_seq_length=64,
                 epochs=30,
                 batch_size=64,
                 use_pretrained=False,
                 pretrained_file=None,
                 save_best_only=True,
                 save_weights_only=False,
                 monitor="val_acc",
                 verbose=1):
        self.training_data = training_data
        self.training_labels = training_labels
        self.val_data = val_data
        self.val_labels = val_labels
        self.word_index = word_index
        self.model_path = model_path
        self.max_seq_length = max_seq_length
        self.epochs = epochs
        self.batch_size = batch_size
        self.save_best_only = save_best_only
        self.save_weights_only = save_weights_only
        self.monitor = monitor
        self.verbose = verbose
        self.use_pretrained = use_pretrained
        self.pretrained_file = pretrained_file

    def create_model(self):
        raise NotImplementedError

    def _encode_and_pad(self):
        self.training_data = tf.keras.preprocessing.sequence.pad_sequences(
            [[self.word_index.get(w, self.word_index["<UNK>"]) for w in line] for line in self.training_data],
            padding='post',
            maxlen=self.max_seq_length, value=self.word_index["<PAD>"]
        )
        self.training_labels = np.asarray(self.training_labels)
        self.val_data = tf.keras.preprocessing.sequence.pad_sequences(
            [[self.word_index.get(w, self.word_index["<UNK>"]) for w in line] for line in self.val_data],
            padding='post',
            maxlen=self.max_seq_length,
            value=self.word_index["<PAD>"]
        )
        self.val_labels = np.asarray(self.val_labels)

    def train(self):
        model = self.create_model()
        model.summary()
        cp_callback = tf.keras.callbacks.ModelCheckpoint(
            filepath=self.model_path,
            save_best_only=self.save_best_only,
            save_weights_only=self.save_weights_only,
            monitor=self.monitor
        )
        self._encode_and_pad()  # 原始数据encode用于输入后续模型
        model.fit(self.training_data,
                  self.training_labels,
                  validation_data=(self.val_data, self.val_labels),
                  epochs=self.epochs,
                  batch_size=self.batch_size,
                  verbose=self.verbose,
                  callbacks=[cp_callback]
                  )



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: __init__.py
@time: 2019/11/25 17:24

这一行开始写关于本文件的说明与解释


"""
from leslie.predictors.text_classifier_predictor import TextBiClassifierPredictor


def func():
    pass


if __name__ == '__main__':
    func()



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: sf_id_intent_slot_model.py
@time: 2020/3/13 15:34

这一行开始写关于本文件的说明与解释


"""
import tensorflow as tf

from leslie.models.model import Model

gpu = tf.config.list_physical_devices('GPU')  # tf2.1版本该函数不再是experimental
print(gpu)  # 前面限定了只使用GPU1(索引是从0开始的,本机有2张RTX2080显卡)
tf.config.experimental.set_memory_growth(gpu[0], True)  # 其实gpus本身就只有一个元素


class SfIdIntentSlotModel(Model):
    def __init__(self,
                 vocab_size,
                 max_seq_length,
                 embedding_size,
                 hidden_size,
                 intent_size,
                 slot_size,
                 use_crf,
                 priority_order="intent_first"):
        super(SfIdIntentSlotModel, self).__init__()
        self.vocab_size = vocab_size
        self.max_seq_length = max_seq_length
        self.embedding_size = embedding_size
        self.hidden_size = hidden_size
        self.intent_size = intent_size
        self.slot_size = slot_size
        self.use_crf = use_crf
        self.priority_order = priority_order

    def call(self, inputs, training=None, mask=None):
        embedded = tf.keras.layers.Embedding(self.vocab_size, self.embedding_size)(inputs)
        # print(embedded.numpy())
        state_outputs, forward_h, forward_c, backward_h, backward_c = tf.keras.layers.Bidirectional(
            tf.keras.layers.LSTM(units=self.hidden_size, return_sequences=True, return_state=True))(embedded)
        # state_outputs 层返回序列结果  (b, s, 2*e); forward_h (b, e)； forward_c (b, e); backward_h  (b, e); backward_c (b, e)
        final_state = tf.concat(values=[forward_h, forward_c, backward_h, backward_c], axis=1)  # [b, 4*e]
        state_shape = state_outputs.get_shape()  # [b, s, 2*e]
        #   ##calculate slot attention
        with tf.name_scope(name="slot_attention"):
            slot_inputs = state_outputs  # [b, s, 2*e]
            attn_size = state_shape[2]  # [2*e]
            origin_shape = tf.shape(state_outputs)  # [b, s, 2*e]
            hidden = tf.expand_dims(state_outputs, 1)  # [b, 1, s, 2*e]
            hidden_convolution = tf.expand_dims(state_outputs, 2)  # [b, s, 1, 2*e]
            k = tf.Variable(name="AttnW",
                            initial_value=tf.initializers.GlorotNormal()(shape=[1, 1, attn_size, attn_size]))
            # [1, 1, 2*e, 2*e]
            hidden_features = tf.nn.conv2d(input=hidden_convolution, filters=k, strides=[1, 1, 1, 1], padding="SAME")
            # [b, s, 1, 2*e]
            hidden_features = tf.reshape(hidden_features, origin_shape)  # [b, s, 2*e]
            hidden_features = tf.expand_dims(hidden_features, 1)  # [b, 1, s, 2*e]
            v = tf.Variable(name="AttnV", initial_value=tf.initializers.GlorotNormal()(shape=[attn_size]))  # [2*e]
            slot_inputs_shape = tf.shape(slot_inputs)  # [b, s, 2*e]
            slot_inputs = tf.reshape(slot_inputs, [-1, attn_size])  # [b*s, 2*e]
            y = tf.keras.layers.Dense(units=attn_size, use_bias=True)(slot_inputs)  # [b*s, 2*e]
            y = tf.reshape(y, slot_inputs_shape)  # [b, s, 2*e]
            y = tf.expand_dims(y, 2)  # [b, s, 1, 2*e]
            s = tf.reduce_sum(v * tf.tanh(hidden_features + y), axis=3)  # [b, s, s]  相加的时候会自动使用广播机制
            a = tf.nn.softmax(s)  # [b, s, s]
            a = tf.expand_dims(a, -1)  # [b, s, s, 1]
            slot_d = tf.reduce_sum(a * hidden, axis=2)  # [b, s, 2*e]
            slot_reinforce_state = tf.expand_dims(slot_d, axis=2)  # [b, s, 1, 2*e]

        #  ##calculate intent attention
        with tf.name_scope(name="intent_attention"):
            intent_input = final_state  # [b, 4*e]
            attn_size = state_shape[2]  # [2*e]
            hidden = tf.expand_dims(state_outputs, 2)  # [b, s, 1, 2*e]
            k = tf.Variable(name="AttnW",
                            initial_value=tf.initializers.GlorotNormal()(shape=[1, 1, attn_size, attn_size]))
            # [1, 1, 2*e, 2*e]
            hidden_features = tf.nn.conv2d(input=hidden, filters=k, strides=[1, 1, 1, 1], padding="SAME")
            # [b, s, 1, 2*e]
            v = tf.Variable(name="AttnV", initial_value=tf.initializers.GlorotNormal()(shape=[attn_size]))  # [2*e]
            y = tf.keras.layers.Dense(units=attn_size, use_bias=True)(intent_input)  # [b, 2*e]
            y = tf.reshape(y, [-1, 1, 1, attn_size])  # [b, 1, 1, 2*e]
            s = tf.reduce_sum(v * tf.tanh(hidden_features + y), axis=[2, 3])  # [b, s]
            a = tf.nn.softmax(s)  # [b, s]
            a = tf.expand_dims(a, axis=-1)  # [b, s, 1]
            a = tf.expand_dims(a, axis=-1)  # [b, s, 1, 1]
            r_intent = tf.reduce_sum(a * hidden, axis=[1, 2])  # [b, 2*e]
            intent_context_states = r_intent  # [b, 2*e]

        if self.priority_order == "intent_first":
            with tf.name_scope(name="intent_subnet"):
                attn_size = state_shape[2]  # [2*e]
                hidden = tf.expand_dims(state_outputs, axis=2)  # [b, s, 1, 2*e]
                k1 = tf.Variable(name="W1",
                                 initial_value=tf.initializers.GlorotNormal()(shape=[1, 1, attn_size, attn_size]))
                # [1, 1, 2*e, 2*e]
                k2 = tf.Variable(name="W2",
                                 initial_value=tf.initializers.GlorotNormal()(shape=[1, 1, attn_size, attn_size]))
                # [1, 1, 2*e, 2*e]
                slot_reinforce_features = tf.nn.conv2d(slot_reinforce_state, filters=k1, strides=[1, 1, 1, 1],
                                                       padding="SAME")
                # [b, s, 1, 2*e]
                hidden_features = tf.nn.conv2d(hidden, filters=k2, strides=[1, 1, 1, 1],
                                               padding="SAME")  # [b, s, 1, 2*e]
                v1 = tf.Variable(name="AttnV", initial_value=tf.initializers.GlorotNormal()(shape=[attn_size]))
                # [s*e]
                bias = tf.Variable(name="Bias", initial_value=tf.initializers.GlorotNormal()(shape=[attn_size]))
                # [2*e]
                s = tf.reduce_sum(v1 * tf.tanh(hidden_features + slot_reinforce_features + bias),
                                  axis=[2, 3])  # [b, s, 1, 2*e]
                a = tf.nn.softmax(s)  # [b, s]
                a = tf.expand_dims(a, axis=-1)  # [b, s, 1]
                a = tf.expand_dims(a, axis=-1)  # [b, s, 1, 1]
                r = tf.reduce_sum(a * slot_reinforce_state, axis=[1, 2])  # [b, 2*e]
                r_intent = r + intent_context_states  # [b, 2*e]
                intent_output = tf.concat([r_intent, intent_input], 1)  # [b, 6*e]

            with tf.name_scope(name="slot_subnet"):
                intent_gate = tf.keras.layers.Dense(units=attn_size, use_bias=True)(r_intent)  # [b, 2*e]
                intent_gate = tf.reshape(intent_gate, [-1, 1, intent_gate.get_shape()[1]])  # [b, 1, 2*e]
                v1 = tf.Variable(name="gateV", initial_value=tf.initializers.GlorotNormal()(shape=[attn_size]))
                # [2*e]
                relation_factor = v1 * tf.tanh(slot_d + intent_gate)  # [b, s, 2*e]
                relation_factor = tf.reduce_sum(relation_factor, axis=2)  # [b, s]
                relation_factor = tf.expand_dims(relation_factor, axis=-1)  # [b, s, 1]
                slot_reinforce_state1 = slot_d + relation_factor  # [b, s, 2*e]
                slot_reinforce_state = tf.expand_dims(slot_reinforce_state1, axis=2)  # [b, s, 2*e, 1]
                slot_reinforce_vector = tf.reshape(slot_reinforce_state1, [-1, attn_size])  # [b*s, 2*e]
                slot_output = tf.concat([slot_reinforce_vector, slot_inputs], 1)  # [b*s, 4*e]
        with tf.name_scope('intent_part'):
            intent = tf.keras.layers.Dense(units=self.intent_size, use_bias=True)(intent_output)
        with tf.name_scope('slot_part'):
            slot = tf.keras.layers.Dense(units=self.slot_size, use_bias=True)(slot_output)
            if self.use_crf:
                slot = tf.reshape(slot, [-1, origin_shape[1], self.slot_size])
        outputs = [slot, intent]
        return outputs

    def train(self):
        pass


if __name__ == '__main__':
    model = SfIdIntentSlotModel(vocab_size=1000, max_seq_length=32, embedding_size=200, hidden_size=200, intent_size=3,
                                slot_size=8, use_crf=True)
    res = model(inputs=tf.constant([[1, 2, 3], [4, 5, 6]]))
    print(res)



# -*- coding: utf-8 -*-
"""
@author: kebo
@contact: kebo0912@outlook.com

@version: 1.0
@file: __init__.py
@time: 2019/12/11 14:48

这一行开始写关于本文件的说明与解释


"""
import argparse
from leslie import __version__
from typing import Any, Optional
from overrides import overrides

from leslie.commands.subcommand import Subcommand


class ArgumentParser(argparse.ArgumentParser):
    """
        Custom argument parser that will display the default value for an argument
        in the help message.
        """

    _action_defaults_to_ignore = {"help"}

    @staticmethod
    def _is_empty_default(default: Any) -> bool:
        if default is None:
            return True
        if isinstance(default, (str, list, tuple, set)):
            return not bool(default)
        return False

    @overrides
    def add_argument(self, *args, **kwargs):
        # Add default value to the help message when the default is meaningful.
        default = kwargs.get("default")
        if kwargs.get(
                "action"
        ) not in self._action_defaults_to_ignore and not self._is_empty_default(default):
            description = kwargs.get("help", "")
            kwargs["help"] = f"{description} (default = {default})"
        super().add_argument(*args, **kwargs)


def create_parser(program):
    """
        Creates the argument parser for the main program.
        """
    parser = ArgumentParser(description="Run Leslie", prog=program)
    parser.add_argument("--version", action="version", version=f"%(prog)s {__version__}")
    subparsers = parser.add_subparsers(title="Commands", metavar="")
    for subcommand_name in sorted(Subcommand.list_available()):
        subcommand_class = Subcommand.by_name(subcommand_name)
        subcommand = subcommand_class()
        subparser = subcommand.add_subparser(subparsers)
        subparser.add_argument(
            "--include-package",
            type=str,
            action="append",
            default=[],
            help="additional packages to include",
        )

    return parser


def main(program):
    parser = create_parser(program)
    args = parser.parse_args()
    if "func" in dir(args):
        args.func(args)
    else:
        parser.print_help()



# # -*- coding: utf-8 -*-
# """
# @author: kebo
# @contact: kebo0912@outlook.com
#
# @version: 1.0
# @file: text_classifier_predictor.py
# @time: 2019/11/25 19:07
#
# 这一行开始写关于本文件的说明与解释
#
#
# """
# import tensorflow as tf
# import pickle
#
#
# class TextBiClassifierPredictor(object):
#     """
#     classifier model predictor
#     """
#
#     def __init__(self, serialization_dir: str):
#         """
#
#         :param serialization_dir:
#         """
#         self.model = tf.keras.models.load_model(serialization_dir)
#         # self.word_index = pickle.load(open(word_index_path, "rb"))
#         # self.max_seq_length = max_seq_length
#         self._token_to_index[namespace] = {}
#         self._index_to_token[namespace] = {}
#
#         with codecs.open(filename, "r", "utf-8") as input_file:
#             lines = input_file.read().split("\n")
#             # Be flexible about having final newline or not
#             if lines and lines[-1] == "":
#                 lines = lines[:-1]
#             for i, line in enumerate(lines):
#                 index = i + 1 if is_padded else i
#                 token = line.replace("@@NEWLINE@@", "\n")
#                 if token == oov_token:
#                     token = self._oov_token
#                 self._token_to_index[namespace][token] = index
#                 self._index_to_token[namespace][index] = token
#
#     def _encode(self, query):
#         """
#         query -> encode_query
#         :param query:
#         :return:
#         """
#         encode_query = [self.word_index.get(w, self.word_index["<UNK>"]) for w in query]
#         return encode_query
#
#     def predict(self, query):
#         """
#         单query predict
#         :param query:
#         :return:
#         """
#         encode_query = self._encode(query=query)
#         encode_query_to_idx = tf.keras.preprocessing.sequence.pad_sequences(
#             [encode_query],
#             padding="post",
#             value=self.word_index["<PAD>"],
#             maxlen=self.max_seq_length
#         )
#         result = self.model.predict(encode_query_to_idx)
#         return result
#
#     def batch_predict(self, query_list):
#         """
#         query batch predict
#         :param query_list:
#         :return:
#         """
#         encode_query_list = [self._encode(query) for query in query_list]
#         encode_query_list_to_idx = tf.keras.preprocessing.sequence.pad_sequences(
#             encode_query_list,
#             padding="post",
#             value=self.word_index["<PAD>"],
#             maxlen=self.max_seq_length
#         )
#         result = self.model.predict(encode_query_list_to_idx)
#         return result
#
#     def evaluate(self, test_data, test_labels):
#         encode_test_data = [self._encode(query) for query in test_data]
#         encode_query_list_to_idx = tf.keras.preprocessing.sequence.pad_sequences(
#             encode_test_data,
#             padding="post",
#             value=self.word_index["<PAD>"],
#             maxlen=self.max_seq_length
#         )
#         import numpy as np
#         encode_test_labels = np.asarray(test_labels)
#         loss, acc = self.model.evaluate(encode_query_list_to_idx, encode_test_labels)
#         return loss, acc
